#!/usr/bin/env python

# Copyright (c) 2006 Jochen Kupperschmidt <webmaster@nwsnet.de>
# Released under the terms of the GNU General Public License
#  _                               _
# | |_ ___ _____ ___ _ _ _ ___ ___| |_
# |   | . |     | ._| | | | . |  _| . /
# |_|_|___|_|_|_|___|_____|___|_| |_|_\
#   http://homework.nwsnet.de/

"""Album Track List Generator - Wizard

A graphical user interface for the Album Track List Generator in wxPython.
"""

__version__ = '14-Apr-2006'


import glob
import os.path
import sys

import wx
import wx.lib.hyperlink as hl
import wx.wizard

from albumtracklist import Album, Template


TEMPLATE_PATH = os.path.abspath('templates')
TEMPLATE_EXT = 'tpl'


class TitledWizardPage(wx.wizard.PyWizardPage):

    def __init__(self, parent, title):
        wx.wizard.PyWizardPage.__init__(self, parent)
        self.prev = self.next = None

        title = wx.StaticText(self, label=title)
        title.SetFont(wx.Font(12, wx.SWISS, wx.NORMAL, wx.BOLD))

        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(title, 0, wx.ALL, 2)
        self.sizer.Add(wx.StaticLine(self), 0, wx.EXPAND|wx.ALL, 2)
        self.SetSizer(self.sizer)

    def GetPrev(self):
        return self.prev

    def GetNext(self):
        return self.next

    def SetPrev(self, prev):
        self.prev = prev

    def SetNext(self, next):
        self.next = next


class TitledPanel(wx.Panel):

    def __init__(self, parent, label):
        wx.Panel.__init__(self, parent)
        box = wx.StaticBox(self, label=label)
        self.sizer = wx.StaticBoxSizer(box)
        self.SetSizer(self.sizer)

    def Add(self, item, proportion=0):
        self.sizer.Add(item, proportion, wx.EXPAND|wx.ALL, 2)


class PathSelector(TitledPanel):

    def __init__(self, parent, label, buttonID):
        TitledPanel.__init__(self, parent, label)
        self.pathLabel = wx.TextCtrl(self, style=wx.TE_READONLY)
        self.button = wx.Button(self, buttonID)
        self.Add(self.pathLabel, 1)
        self.Add(self.button)

# ---------------------------------------------------------------- #

class AlbumPanel(TitledPanel):

    def __init__(self, parent):
        TitledPanel.__init__(self, parent, 'Album')
        self.details = wx.TextCtrl(self,
            style=wx.TE_MULTILINE|wx.TE_DONTWRAP|wx.TE_READONLY)
        self.details.WriteText('No album loaded.')
        self.Add(self.details, 1)


class AlbumSelectionPage(TitledWizardPage):

    def __init__(self, parent):
        TitledWizardPage.__init__(self, parent, title='Step 1: Choose an album')

        self.path = None
        self.album = None

        self.pathPanel = PathSelector(self, 'Directory', wx.ID_OPEN)
        self.sizer.Add(self.pathPanel, 0, wx.EXPAND|wx.ALL, 2)

        self.albumPanel = AlbumPanel(self)
        self.sizer.Add(self.albumPanel, 1, wx.EXPAND|wx.ALL, 2)

        self.pathPanel.button.Bind(wx.EVT_BUTTON, self.loadAlbum)

    def GetNext(self):
        if self.album is None:
            return self
        else:
            self.next.album = self.album
            return self.next

    def loadAlbum(self, evt):
        """Read album details."""
        dlg = wx.DirDialog(self, 'Choose a directory:',
            style=wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER)
        if self.path is not None:
            dlg.SetPath(self.path)
        returnCode = dlg.ShowModal()
        self.path = dlg.GetPath()
        dlg.Destroy()
        if returnCode != wx.ID_OK:
            return
        self.pathPanel.pathLabel.SetValue(self.path)

        self.album = Album.fromPath(self.path)
        self.albumPanel.details.Clear()
        if self.album is None:
            details = 'Could not recognize album.'
        else:
            details = self.album.getDetails()
        self.albumPanel.details.WriteText(details)

# ---------------------------------------------------------------- #

class TemplateSelector(TitledPanel):

    def __init__(self, parent):
        TitledPanel.__init__(self, parent, 'Template')

        # Get a list of available templates.
        templates = sorted(map(os.path.basename,
            glob.glob(os.path.join(TEMPLATE_PATH, '*.%s' % TEMPLATE_EXT))))

        self.choice = wx.Choice(self, choices=templates)
        self.choice.SetSelection(0)
        self.Add(self.choice, 1)


class TemplateSelectionPage(TitledWizardPage):

    def __init__(self, parent):
        TitledWizardPage.__init__(self, parent, title='Step 2: Create the cover')

        self.lastSavePath = None

        self.tplSel = TemplateSelector(self)
        self.sizer.Add(self.tplSel, 0, wx.EXPAND|wx.ALL, 2)

        self.pathPanel = PathSelector(self, 'Filename', wx.ID_SAVEAS)
        self.sizer.Add(self.pathPanel, 0, wx.EXPAND|wx.ALL, 2)

        self.pathPanel.button.Bind(wx.EVT_BUTTON, self.writeCover)

    def writeCover(self, evt):
        """Compile and save album cover."""
        path = self.showSaveDlg()
        if not path:
            return
        t = Template(os.path.join(TEMPLATE_PATH,
            self.tplSel.choice.GetStringSelection()))
        f = open(path, 'w')
        t.expand(self.album, f)
        f.close()

    def showSaveDlg(self):
        """Ask the user for a filename so save to."""
        if not self.album:
            raise StandardError, 'No album loaded.'
        dlg = wx.FileDialog(self, 'Choose a filename:',
            style=wx.SAVE|wx.OVERWRITE_PROMPT)
        if self.lastSavePath is not None:
            dlg.SetPath(self.lastSavePath)
        dlg.SetFilename('%(artist)s - %(title)s.html' % self.album.__dict__)
        path = None
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            self.lastSavePath = path
        dlg.Destroy()
        self.pathPanel.pathLabel.SetValue(path)
        self.next.setCoverPath(path)
        return path

# ---------------------------------------------------------------- #

class CoverCreationPage(TitledWizardPage):

    def __init__(self, parent):
        TitledWizardPage.__init__(self, parent, title='Step 3: View the cover')

        self.hl = hl.HyperLinkCtrl(self, label='Open cover in browser')
        self.sizer.Add(self.hl, 0, wx.EXPAND|wx.ALL, 2)

    def setCoverPath(self, path):
        self.hl.SetURL('file:///' + path.replace('\\', '/'))

# ---------------------------------------------------------------- #

if __name__ == '__main__':
    # Create wizard.
    app = wx.PySimpleApp()
    wiz = wx.wizard.Wizard(None, title='Album Cover Wizard')

    # Setup and connect pages.
    page1 = AlbumSelectionPage(wiz)
    page2 = TemplateSelectionPage(wiz)
    page3 = CoverCreationPage(wiz)
    page1.SetNext(page2)
    page2.SetPrev(page1)
    page2.SetNext(page3)
    page3.SetPrev(page2)

    # Run wizard.
    if wiz.RunWizard(page1):
        print 'Wizard completed'
    else:
        print 'Wizard cancelled.'
